@@ -1,9 +1,11 @@
 use std::collections::HashMap;
 
-use quote::format_ident;
+use syn::ext::IdentExt;
+use syn::Ident;
 
 use crate::ir::SavvyMergedImpl;
-use crate::{MergedResult, SavvyEnum, SavvyFn, SavvyFnType};
+use crate::utils::add_indent;
+use crate::{MergedResult, SavvyEnum, SavvyFn, SavvyFnArg, SavvyFnType};
 
 use crate::ir::savvy_fn::{SavvyFnReturnType, UserDefinedStructReturnType};
 
@@ -14,49 +16,109 @@ fn get_r_doc_comment(docs: &[String]) -> String {
         .join("\n")
 }
 
+fn ident_to_str_r(x: &Ident) -> String {
+    x.unraw().to_string()
+}
+
+impl SavvyEnum {
+    pub fn class_name_r(&self) -> String {
+        self.ty.unraw().to_string()
+    }
+}
+
+impl SavvyFnArg {
+    pub fn arg_name_r(&self) -> String {
+        self.pat.unraw().to_string()
+    }
+}
+
 impl SavvyFn {
-    pub fn fn_name_r(&self) -> syn::Ident {
+    pub fn fn_name_r(&self) -> String {
+        self.name_r(true)
+    }
+
+    pub fn field_name_r(&self) -> String {
+        self.name_r(false)
+    }
+
+    fn name_r(&self, with_ty: bool) -> String {
+        let fn_name = ident_to_str_r(&self.fn_name);
+
+        if !with_ty {
+            return fn_name;
+        }
+
         match self.get_self_ty_ident() {
             Some(ty) => {
+                let ty_name = ident_to_str_r(&ty);
+
                 // Special convention. If the method name is "new", use type
                 // itself.
-                if self.fn_name.to_string().as_str() == "new" {
-                    ty
+                if fn_name.as_str() == "new" {
+                    ty_name
                 } else {
-                    format_ident!("{}_{}", ty, self.fn_name)
+                    format!("{}_{}", ty, fn_name)
                 }
             }
-            None => self.fn_name.clone(),
+            None => fn_name,
         }
     }
 
-    fn get_r_args(&self) -> Vec<String> {
-        let mut out: Vec<_> = self.args.iter().map(|arg| arg.pat_string()).collect();
+    // The return value is (pat, default value)
+    fn get_r_args(&self) -> Vec<(String, Option<String>)> {
+        let mut out: Vec<_> = self
+            .args
+            .iter()
+            .map(|arg| {
+                let arg_name = arg.arg_name_r();
+                let default_value = if arg.is_optional() {
+                    Some("NULL".to_string())
+                } else {
+                    None
+                };
+
+                (arg_name, default_value)
+            })
+            .collect();
 
         // if it's a method, add `self__` arg
         if matches!(&self.fn_type, SavvyFnType::Method { .. }) {
-            out.insert(0, "self".to_string())
+            out.insert(0, ("self".to_string(), None))
         }
 
         out
     }
 
+    fn get_r_args_for_signature(&self) -> Vec<String> {
+        self.get_r_args()
+            .iter()
+            .map(|(arg_name, default_value)| {
+                if let Some(value) = default_value {
+                    format!("`{arg_name}` = {value}")
+                } else {
+                    format!("`{arg_name}`")
+                }
+            })
+            .collect::<Vec<_>>()
+    }
+
+    fn get_r_args_for_call(&self) -> Vec<String> {
+        self.get_r_args()
+            .iter()
+            .map(|(arg_name, _)| format!("`{arg_name}`"))
+            .collect::<Vec<_>>()
+    }
+
     fn to_r_function(&self) -> String {
         let fn_name = self.fn_name_r();
         let fn_name_c = self.fn_name_c_impl();
 
         let doc_comments = get_r_doc_comment(self.docs.as_slice());
 
-        let args = self
-            .get_c_args()
-            .iter()
-            .map(|(pat, _)| pat.clone())
-            .collect::<Vec<String>>();
+        let args_sig = self.get_r_args_for_signature().join(", ");
 
-        let mut args_call = args.clone();
+        let mut args_call = self.get_r_args_for_call();
         args_call.insert(0, format!("{fn_name_c}"));
-
-        let args = args.join(", ");
         let args_call = args_call.join(", ");
 
         let mut body_lines = self.get_extractions();
@@ -64,22 +126,21 @@ impl SavvyFn {
         // If the result is NULL, wrap it with invisible
         let fn_call = match &self.return_type {
             SavvyFnReturnType::Unit(_) => {
-                format!("  invisible(.Call({args_call}))")
+                format!("invisible(.Call({args_call}))")
             }
-            SavvyFnReturnType::Sexp(_) => format!("  .Call({args_call})"),
-            SavvyFnReturnType::UserDefinedStruct(UserDefinedStructReturnType {
-                ty_str, ..
-            }) => {
-                format!("  .savvy_wrap_{ty_str}(.Call({args_call}))")
+            SavvyFnReturnType::Sexp(_) => format!(".Call({args_call})"),
+            SavvyFnReturnType::UserDefinedStruct(UserDefinedStructReturnType { ty, .. }) => {
+                let ty_str = ident_to_str_r(ty);
+                format!(".savvy_wrap_{ty_str}(.Call({args_call}))")
             }
         };
         body_lines.push(fn_call);
 
-        let body = body_lines.join("\n");
+        let body = add_indent(&body_lines.join("\n"), 2);
 
         format!(
             "{doc_comments}
-{fn_name} <- function({args}) {{
+`{fn_name}` <- function({args_sig}) {{
 {body}
 }}
 "
@@ -94,10 +155,10 @@ impl SavvyFn {
                     return None;
                 }
 
-                let r_var = arg.pat_string();
+                let r_var = arg.arg_name_r();
                 let r_class = arg.ty_string();
                 Some(format!(
-                    r#"  {r_var} <- .savvy_extract_ptr({r_var}, "{r_class}")"#
+                    r#"`{r_var}` <- .savvy_extract_ptr(`{r_var}`, "{r_class}")"#
                 ))
             })
             .collect::<Vec<String>>()
@@ -106,12 +167,13 @@ impl SavvyFn {
 
 fn generate_r_impl_for_impl(
     i: &SavvyMergedImpl,
-    ty: &str,
-    enum_types: &HashMap<String, &SavvyEnum>,
+    ty: &Ident,
+    enum_types: &HashMap<Ident, &SavvyEnum>,
 ) -> String {
     let mut associated_fns: Vec<&SavvyFn> = Vec::new();
     let mut method_fns: Vec<&SavvyFn> = Vec::new();
-    let class_r = ty;
+    let class_r = ident_to_str_r(ty);
+    let class_r_for_bundle = format!("{class_r}__bundle");
 
     for savvy_fn in &i.fns {
         match savvy_fn.fn_type {
@@ -127,17 +189,17 @@ fn generate_r_impl_for_impl(
             let fn_name = x.fn_name_r();
             let fn_name_c = x.fn_name_c_impl();
 
-            let mut args = x.get_r_args();
             // Remove self from arguments for R
-            let args_r = args
-                .clone()
+            let args_sig = x
+                .get_r_args_for_signature()
                 .into_iter()
-                .filter(|e| *e != "self")
+                .filter(|e| *e != "`self`")
                 .collect::<Vec<String>>()
                 .join(", ");
 
-            args.insert(0, format!("{fn_name_c}"));
-            let args_call = args.join(", ");
+            let mut args_call = x.get_r_args_for_call();
+            args_call.insert(0, format!("{fn_name_c}"));
+            let args_call = args_call.join(", ");
 
             let mut body_lines = x.get_extractions();
 
@@ -148,20 +210,20 @@ fn generate_r_impl_for_impl(
                 // If the result is an external pointer, wrap it with the
                 // corresponding wraping function
                 SavvyFnReturnType::UserDefinedStruct(UserDefinedStructReturnType {
-                    ty_str,
-                    ..
+                    ty, ..
                 }) => {
-                    format!("  .savvy_wrap_{ty_str}(.Call({args_call}))")
+                    let ty_str = ident_to_str_r(ty);
+                    format!(".savvy_wrap_{ty_str}(.Call({args_call}))")
                 }
             };
             body_lines.push(fn_call);
 
-            let body = body_lines.join("\n");
+            let body = add_indent(&body_lines.join("\n"), 4);
 
             format!(
-                "{fn_name} <- function(self) {{
-  function({args_r}) {{
-  {body}
+                "`{fn_name}` <- function(self) {{
+  function({args_sig}) {{
+{body}
   }}
 }}
 "
@@ -172,36 +234,41 @@ fn generate_r_impl_for_impl(
 
     let methods = method_fns
         .iter()
-        .map(|o| format!("  e${} <- {}(ptr)", o.fn_name, o.fn_name_r()))
+        .map(|o| format!("  e$`{}` <- `{}`(ptr)", o.field_name_r(), o.fn_name_r()))
         .collect::<Vec<String>>()
         .join("\n");
 
     let wrap_fn_name = format!(".savvy_wrap_{}", class_r);
 
     let wrap_fn = format!(
-        r#"{wrap_fn_name} <- function(ptr) {{
+        r#"`{wrap_fn_name}` <- function(ptr) {{
   e <- new.env(parent = emptyenv())
   e$.ptr <- ptr
-  {methods}
-  
+{methods}
+
   class(e) <- "{class_r}"
   e
 }}
+
+#' @export
+`$<-.{class_r}` <- function(x, name, value) stop("{class_r} cannot be modified", call. = FALSE)
+
+#' @export
+`[[<-.{class_r}` <- function(x, i, value) stop("{class_r} cannot be modified", call. = FALSE)
 "#
     );
 
     let associated_fns = associated_fns
         .iter()
         .map(|x| {
-            let fn_name = x.fn_name.clone();
+            let fn_name = x.field_name_r();
             let fn_name_c = x.fn_name_c_impl();
 
-            let mut args = x.get_r_args();
+            let args_sig = x.get_r_args_for_signature().join(", ");
 
-            let args_r = args.join(", ");
-
-            args.insert(0, format!("{fn_name_c}"));
-            let args_call = args.join(", ");
+            let mut args_call = x.get_r_args_for_call();
+            args_call.insert(0, format!("{fn_name_c}"));
+            let args_call = args_call.join(", ");
 
             let mut body_lines = x.get_extractions();
 
@@ -212,19 +279,19 @@ fn generate_r_impl_for_impl(
                 // If the result is an external pointer, wrap it with the
                 // corresponding wraping function
                 SavvyFnReturnType::UserDefinedStruct(UserDefinedStructReturnType {
-                    ty_str,
-                    ..
+                    ty, ..
                 }) => {
-                    format!("  .savvy_wrap_{ty_str}(.Call({args_call}))")
+                    let ty_str = ident_to_str_r(ty);
+                    format!(".savvy_wrap_{ty_str}(.Call({args_call}))")
                 }
             };
 
             body_lines.push(fn_call);
 
-            let body = body_lines.join("\n");
+            let body = add_indent(&body_lines.join("\n"), 2);
 
             format!(
-                r#"{class_r}${fn_name} <- function({args_r}) {{
+                r#"`{class_r}`$`{fn_name}` <- function({args_sig}) {{
 {body}
 }}
 "#
@@ -235,13 +302,13 @@ fn generate_r_impl_for_impl(
 
     let init = match enum_types.get(ty) {
         Some(e) => generate_r_impl_for_enum(e),
-        None => format!("{class_r} <- new.env(parent = emptyenv())"),
+        None => format!("`{class_r}` <- new.env(parent = emptyenv())"),
     };
 
     let doc_comments = get_r_doc_comment(i.docs.as_slice());
 
     format!(
-        "### wrapper functions for {class_r}
+        r#"### wrapper functions for {class_r}
 
 {closures}
 {wrap_fn}
@@ -249,37 +316,82 @@ fn generate_r_impl_for_impl(
 {doc_comments}
 {init}
 
+#' @export
+`$<-.{class_r}` <- function(x, name, value) stop("{class_r} cannot be modified", call. = FALSE)
+
+#' @export
+`[[<-.{class_r}` <- function(x, i, value) stop("{class_r} cannot be modified", call. = FALSE)
+
 ### associated functions for {class_r}
 
 {associated_fns}
-"
+
+class(`{class_r}`) <- "{class_r_for_bundle}"
+
+#' @export
+`print.{class_r_for_bundle}` <- function(x, ...) {{
+  cat('{class_r}')
+}}
+
+#' @export
+`$<-.{class_r_for_bundle}` <- function(x, name, value) stop("{class_r} cannot be modified", call. = FALSE)
+
+#' @export
+`[[<-.{class_r_for_bundle}` <- function(x, i, value) stop("{class_r} cannot be modified", call. = FALSE)
+"#
     )
 }
 
 fn generate_r_impl_for_enum(e: &SavvyEnum) -> String {
-    let class_r = e.ty.to_string();
+    let class_r = e.class_name_r();
+    let class_r_for_bundle = format!("{class_r}__bundle");
 
     let variants = e
         .variants
         .iter()
         .enumerate()
-        .map(|(i, v)| format!(r#"{class_r}${v} <- .savvy_wrap_{class_r}({i}L)"#))
+        .map(|(i, v)| {
+            let v = ident_to_str_r(v);
+            format!(r#"`{class_r}`$`{v}` <- .savvy_wrap_{class_r}({i}L)"#)
+        })
         .collect::<Vec<String>>()
         .join("\n");
 
     let variant_labels = e
         .variants
         .iter()
-        .map(|x| format!(r#""{x}""#))
+        .map(|x| format!(r#""{}""#, ident_to_str_r(x)))
         .collect::<Vec<String>>()
         .join(", ");
 
     format!(
-        r#"{class_r} <- new.env(parent = emptyenv())
+        r#"`{class_r}` <- new.env(parent = emptyenv())
 {variants}
 
 #' @export
-print.{class_r} <- function(x, ...) {{
+`$.{class_r_for_bundle}` <- function(x, name) {{
+  if (!name %in% c({variant_labels})) {{
+    stop(paste0("Unknown variant: ", name), call. = FALSE)
+  }}
+
+  NextMethod()
+}}
+
+#' @export
+`[[.{class_r_for_bundle}` <- function(x, i) {{
+  if (is.numeric(i)) {{
+    stop("{class_r} cannot be subset by index", call. = FALSE)
+  }}
+
+  if (!i %in% c({variant_labels})) {{
+    stop(paste0("Unknown variant: ", i), call. = FALSE)
+  }}
+
+  NextMethod()
+}}
+
+#' @export
+`print.{class_r}` <- function(x, ...) {{
   idx <- x$.ptr + 1L
   label <- c({variant_labels})[idx]
   if (is.na(label)) {{
@@ -292,8 +404,8 @@ print.{class_r} <- function(x, ...) {{
 }
 
 pub fn generate_r_impl_file(result: &MergedResult, pkg_name: &str) -> String {
-    let enum_types: HashMap<String, &SavvyEnum> =
-        result.enums.iter().map(|e| (e.ty.to_string(), e)).collect();
+    let enum_types: HashMap<Ident, &SavvyEnum> =
+        result.enums.iter().map(|e| (e.ty.clone(), e)).collect();
 
     let r_fns = result
         .bare_fns
@@ -325,6 +437,10 @@ NULL
 
 # Check class and extract the external pointer embedded in the environment
 .savvy_extract_ptr <- function(e, class) {{
+  if(is.null(e)) {{
+    return(NULL)
+  }}
+
   if(inherits(e, class)) {{
     e$.ptr
   }} else {{
