@@ -1,9 +1,11 @@
+use proc_macro2::Span;
 use quote::format_ident;
-use syn::{parse_quote, Attribute, FnArg::Typed, Pat::Ident, PatType, Signature, Stmt};
+use syn::{ext::IdentExt, parse_quote, Attribute, FnArg::Typed, Pat::Ident, PatType, Signature, Stmt};
 
 use crate::utils::extract_docs;
 
 enum SavvyInputTypeCategory {
+    Sexp,
     SexpWrapper,
     PrimitiveType,
     UserDefinedTypeRef, // &T
@@ -15,11 +17,12 @@ struct SavvyInputType {
     category: SavvyInputTypeCategory,
     ty_orig: syn::Type,
     ty_str: String,
+    optional: bool,
 }
 
 #[allow(dead_code)]
 impl SavvyInputType {
-    fn from_type(ty: &syn::Type) -> syn::Result<Self> {
+    fn from_type(ty: &syn::Type, in_option: bool) -> syn::Result<Self> {
         match &ty {
             // User-defined structs are accepted in the form of either `&` or
             // `&mut`. Note that `&str` also falls here.
@@ -31,12 +34,14 @@ impl SavvyInputType {
                             category: SavvyInputTypeCategory::PrimitiveType,
                             ty_orig: ty.clone(),
                             ty_str: "&str".to_string(),
+                            optional: in_option,
                         })
                     } else {
                         Ok(Self {
                             category: SavvyInputTypeCategory::UserDefinedTypeRef,
                             ty_orig: ty.clone(),
                             ty_str,
+                            optional: in_option,
                         })
                     }
                 } else {
@@ -48,12 +53,38 @@ impl SavvyInputType {
             }
 
             syn::Type::Path(type_path) => {
-                let type_ident = &type_path.path.segments.last().unwrap().ident;
+                let type_path_last = type_path.path.segments.last().unwrap();
+                let type_ident = &type_path_last.ident;
                 let ty_str = type_ident.to_string();
                 match ty_str.as_str() {
+                    "Option" => {
+                        if in_option {
+                            return Err(syn::Error::new_spanned(
+                                type_path,
+                                "`Option` cannot be nested",
+                            ));
+                        }
+
+                        if let syn::PathArguments::AngleBracketed(
+                            syn::AngleBracketedGenericArguments { args, .. },
+                        ) = &type_path_last.arguments
+                        {
+                            if args.len() == 1 {
+                                if let syn::GenericArgument::Type(ty) = &args.first().unwrap() {
+                                    return Self::from_type(ty, true);
+                                }
+                            }
+                        }
+
+                        Err(syn::Error::new_spanned(
+                            type_path,
+                            "Option<T> can accept only a type",
+                        ))
+                    }
+
                     // Owned-types are not allowed for the input
                     "OwnedIntegerSexp" | "OwnedRealSexp" | "OwnedComplexSexp"
-                    | "OwnedLogicalSexp" | "OwnedStringSexp" | "OwnedListSexp" => {
+                    | "OwnedLogicalSexp" | "OwnedRawSexp" | "OwnedStringSexp" | "OwnedListSexp" => {
                         let msg = format!(
                             "`Owned-` types are not allowed here. Did you mean `{}`?",
                             ty_str.strip_prefix("Owned").unwrap()
@@ -61,19 +92,31 @@ impl SavvyInputType {
                         Err(syn::Error::new_spanned(type_path, msg))
                     }
 
+                    // Since Sexp doesn't need to be converted by try_from(),
+                    // this needs to be handled separately.
+                    "Sexp" => Ok(Self {
+                        category: SavvyInputTypeCategory::Sexp,
+                        ty_orig: ty.clone(),
+                        ty_str,
+                        optional: in_option,
+                    }),
+
                     // Read-only types
-                    "Sexp" | "IntegerSexp" | "RealSexp" | "NumericSexp" | "ComplexSexp"
-                    | "LogicalSexp" | "StringSexp" | "ListSexp" | "FunctionSexp" => Ok(Self {
+                    "IntegerSexp" | "RealSexp" | "NumericSexp" | "ComplexSexp"
+                    | "LogicalSexp" | "RawSexp" | "StringSexp" | "ListSexp" | "FunctionSexp"
+                    | "EnvironmentSexp" => Ok(Self {
                         category: SavvyInputTypeCategory::SexpWrapper,
                         ty_orig: ty.clone(),
                         ty_str,
+                        optional: in_option,
                     }),
 
                     // Primitive types
-                    "i32" | "usize" | "f64" | "bool" | "NumericScalar" => Ok(Self {
+                    "i32" | "usize" | "f64" | "bool" | "u8" | "NumericScalar" => Ok(Self {
                         category: SavvyInputTypeCategory::PrimitiveType,
                         ty_orig: ty.clone(),
                         ty_str,
+                        optional: in_option,
                     }),
 
                     "DllInfo" => Err(syn::Error::new_spanned(
@@ -85,6 +128,7 @@ impl SavvyInputType {
                         category: SavvyInputTypeCategory::UserDefinedType,
                         ty_orig: ty.clone(),
                         ty_str,
+                        optional: in_option,
                     }),
                 }
             }
@@ -117,6 +161,7 @@ impl SavvyInputType {
                     category: SavvyInputTypeCategory::DllInfo,
                     ty_orig: ty.clone(),
                     ty_str: type_ident.to_string(),
+                    optional: in_option,
                 })
             }
 
@@ -152,7 +197,7 @@ impl SavvyInputType {
 }
 
 pub struct SavvyFnArg {
-    pat: syn::Ident,
+    pub(crate) pat: syn::Ident,
     ty: SavvyInputType,
 }
 
@@ -168,14 +213,14 @@ impl SavvyFnArg {
         )
     }
 
-    pub fn pat_string(&self) -> String {
-        self.pat.to_string()
-    }
-
     pub fn ty_string(&self) -> String {
         self.ty.ty_str.clone()
     }
 
+    pub fn is_optional(&self) -> bool {
+        self.ty.optional
+    }
+
     pub fn to_c_type_string(&self) -> String {
         self.ty.to_c_type()
     }
@@ -195,7 +240,7 @@ impl SavvyFnArg {
 /// - `savvy::Result<Self>`
 /// - `Self`
 pub struct UserDefinedStructReturnType {
-    pub(crate) ty_str: String,
+    pub(crate) ty: syn::Ident,
     pub(crate) return_type: syn::ReturnType,
     pub(crate) wrapped_with_result: bool,
 }
@@ -356,32 +401,63 @@ impl SavvyFn {
                         }
                     };
 
-                    let ty = match SavvyInputType::from_type(ty.as_ref()) {
+                    let ty = match SavvyInputType::from_type(ty.as_ref(), false) {
                         Ok(ty) => ty,
                         Err(e) => return Some(Err(e)),
                     };
                     let ty_ident = ty.to_rust_type_outer();
 
-                    match fn_type {
+                    match (&fn_type, &ty.category) {
                         // DllInfo is passed as it is
-                        SavvyFnType::InitFunction => {
-                            if !matches!(ty.category, SavvyInputTypeCategory::DllInfo) {
-                                return Some(Err(syn::Error::new_spanned(
-                                    ty.ty_orig,
-                                    "#[savvy_init] can be used only on a function that takes `*mut DllInfo`",
-                                )));
-                            }
+                        (&SavvyFnType::InitFunction, &SavvyInputTypeCategory::DllInfo) => {}
+                        
+                        (&SavvyFnType::InitFunction, _) => {
+                            return Some(Err(syn::Error::new_spanned(
+                                ty.ty_orig,
+                                "#[savvy_init] can be used only on a function that takes `*mut DllInfo`",
+                            )));
                         }
-                        _ => {
-                            if matches!(ty.category, SavvyInputTypeCategory::DllInfo) {
+
+                        (_, &SavvyInputTypeCategory::DllInfo) => {
                                 return Some(Err(syn::Error::new_spanned(
                                     ty.ty_orig,
                                     "#[savvy] doesn't accept `*mut DllInfo`. Did you mean #[savvy_init]?",
                                 )));
+                        }
+
+                        (_, &SavvyInputTypeCategory::Sexp) => {
+                            if ty.optional {
+                                stmts_additional.push(parse_quote! { let #pat = savvy::Sexp(#pat); });
+                                stmts_additional.push(parse_quote! { 
+                                    let #pat = if #pat.is_null() { 
+                                        None
+                                     } else { 
+                                        Some(#pat)
+                                    };
+                                })
+                            } else {
+                                stmts_additional.push(parse_quote! {
+                                    let #pat = savvy::Sexp(#pat);
+                                });
+                            }
+                        }
+
+                        (_, _) => {
+                            let arg_lit = syn::LitStr::new(&pat.unraw().to_string(), Span::call_site());
+                            if ty.optional {
+                                stmts_additional.push(parse_quote! { let #pat = savvy::Sexp(#pat); });
+                                stmts_additional.push(parse_quote! { 
+                                    let #pat = if #pat.is_null() { 
+                                        None
+                                     } else { 
+                                        Some(<#ty_ident>::try_from(#pat).map_err(|e| e.with_arg_name(#arg_lit))?) 
+                                    };
+                                })
+                            } else {
+                                stmts_additional.push(parse_quote! {
+                                    let #pat = <#ty_ident>::try_from(savvy::Sexp(#pat)).map_err(|e| e.with_arg_name(#arg_lit))?;
+                                });
                             }
-                            stmts_additional.push(parse_quote! {
-                                let #pat = <#ty_ident>::try_from(savvy::Sexp(#pat))?;
-                            });
                         }
                     }
 
@@ -392,6 +468,15 @@ impl SavvyFn {
             })
             .collect::<syn::Result<Vec<SavvyFnArg>>>()?;
 
+        // reject signature like fn (x: Option<i32>, y: i32)
+        let mut args_after_optional = args_new.iter().skip_while(|x| !x.is_optional());
+        if args_after_optional.any(|x| !x.is_optional()) {
+            return Err(syn::Error::new_spanned(
+                sig.inputs.clone(),
+                "optional args can be placed only after mandatory args",
+            ));
+        }
+
         // Check for init function
         let is_init_fn = args_new
             .iter()
@@ -421,9 +506,9 @@ impl SavvyFn {
     }
 }
 
-fn self_ty_to_string(self_ty: Option<&syn::Type>) -> Option<String> {
+fn self_ty_to_actual_ty(self_ty: Option<&syn::Type>) -> Option<syn::Ident> {
     if let Some(syn::Type::Path(type_path)) = self_ty {
-        Some(type_path.path.segments.last().unwrap().ident.to_string())
+        Some(type_path.path.segments.last().unwrap().ident.clone())
     } else {
         None
     }
@@ -444,6 +529,7 @@ fn get_savvy_return_type(
             return_type.clone(),
             "function must have return type",
         )),
+
         syn::ReturnType::Type(_, ty) => {
             let e = Err(syn::Error::new_spanned(
                 return_type.clone(),
@@ -462,18 +548,18 @@ fn get_savvy_return_type(
                     let last_path_seg = type_path.path.segments.last().unwrap();
                     match (
                         last_path_seg.ident.to_string().as_str(),
-                        self_ty_to_string(self_ty),
+                        self_ty_to_actual_ty(self_ty),
                     ) {
                         // if Result, do further investigation about hte inside.
                         ("Result", _) => {}
                         // if Self or the same as the self type, it's allowed
-                        (ret_ty_str, Some(ty_str)) => {
-                            if ret_ty_str != "Self" && ret_ty_str != ty_str {
+                        (ret_ty_str, Some(ty_actual)) => {
+                            if ret_ty_str != "Self" && ty_actual != ret_ty_str {
                                 return e;
                             } else {
                                 return Ok(SavvyFnReturnType::UserDefinedStruct(
                                     UserDefinedStructReturnType {
-                                        ty_str,
+                                        ty: ty_actual,
                                         return_type: parse_quote!(-> savvy::Result<#self_ty>),
                                         wrapped_with_result: false,
                                     },
@@ -490,63 +576,61 @@ fn get_savvy_return_type(
             };
 
             // Check `T`` in savvy::Result<T>
-            match path_args {
-                syn::PathArguments::AngleBracketed(syn::AngleBracketedGenericArguments {
-                    args,
-                    ..
-                }) => {
-                    if args.len() != 1 {
-                        return e;
-                    }
-                    match &args.first().unwrap() {
-                        syn::GenericArgument::Type(ty) => match ty {
-                            syn::Type::Tuple(type_tuple) => {
-                                if type_tuple.elems.is_empty() {
-                                    Ok(SavvyFnReturnType::Unit(return_type.clone()))
-                                } else {
-                                    e
-                                }
-                            }
-                            syn::Type::Path(type_path) => {
-                                let ty_str =
-                                    type_path.path.segments.last().unwrap().ident.to_string();
-                                match ty_str.as_str() {
-                                    "Sexp" => Ok(SavvyFnReturnType::Sexp(return_type.clone())),
 
-                                    // if it's `savvy::Result<Self>`, replace `Self` with the actual type
-                                    "Self" => {
-                                        let ty_str =
-                                            if let Some(ty_str) = self_ty_to_string(self_ty) {
-                                                ty_str
-                                            } else {
-                                                return e;
-                                            };
-                                        Ok(SavvyFnReturnType::UserDefinedStruct(
+            if let syn::PathArguments::AngleBracketed(syn::AngleBracketedGenericArguments {
+                args,
+                ..
+            }) = path_args
+            {
+                if args.len() != 1 {
+                    return e;
+                }
+
+                if let syn::GenericArgument::Type(ty) = &args.first().unwrap() {
+                    match ty {
+                        syn::Type::Tuple(type_tuple) => {
+                            if type_tuple.elems.is_empty() {
+                                return Ok(SavvyFnReturnType::Unit(return_type.clone()));
+                            }
+                        }
+
+                        syn::Type::Path(type_path) => {
+                            let last_ident = &type_path.path.segments.last().unwrap().ident;
+                            match last_ident.to_string().as_str() {
+                                "Sexp" => return Ok(SavvyFnReturnType::Sexp(return_type.clone())),
+
+                                // if it's `savvy::Result<Self>`, replace `Self` with the actual type
+                                "Self" => {
+                                    if let Some(ty_actual) = self_ty_to_actual_ty(self_ty) {
+                                        return Ok(SavvyFnReturnType::UserDefinedStruct(
                                             UserDefinedStructReturnType {
-                                                ty_str,
+                                                ty: ty_actual,
                                                 return_type: parse_quote!(-> savvy::Result<#self_ty>),
                                                 wrapped_with_result: true,
                                             },
-                                        ))
+                                        ));
                                     }
+                                }
 
-                                    // if it's the actual type, us as it is.
-                                    _ => Ok(SavvyFnReturnType::UserDefinedStruct(
+                                // if it's the actual type, use it as it is.
+                                _ => {
+                                    return Ok(SavvyFnReturnType::UserDefinedStruct(
                                         UserDefinedStructReturnType {
-                                            ty_str,
+                                            ty: last_ident.clone(),
                                             return_type: return_type.clone(),
                                             wrapped_with_result: true,
                                         },
-                                    )),
+                                    ))
                                 }
                             }
-                            _ => e,
-                        },
-                        _ => e,
+                        }
+
+                        _ => {}
                     }
                 }
-                _ => e,
             }
+
+            e
         }
     }
 }
@@ -631,7 +715,7 @@ mod tests {
             let srt = get_savvy_return_type(rt, Some(&self_ty));
             assert!(srt.is_ok());
             if let SavvyFnReturnType::UserDefinedStruct(uds) = srt.unwrap() {
-                assert_eq!(uds.ty_str, "Foo");
+                assert_eq!(uds.ty.to_string().as_str(), "Foo");
                 assert_eq!(uds.return_type, parse_quote!(-> savvy::Result<Foo>));
                 assert_eq!(uds.wrapped_with_result, i != 0); // only the first case is false
             } else {
