@@ -36,41 +36,49 @@
 //!
 //! - Compile PRQL queries to SQL at run time.
 //!
-//!     ```
-//!     # fn main() -> Result<(), prqlc::ErrorMessages> {
-//!     let sql = prqlc::compile(
-//!         "from albums | select {title, artist_id}",
-//!          &prqlc::Options::default().no_format()
-//!     )?;
-//!     assert_eq!(&sql[..35], "SELECT title, artist_id FROM albums");
-//!     # Ok(())
-//!     # }
-//!     ```
+//!   ```
+//!   # fn main() -> Result<(), prqlc::ErrorMessages> {
+//!   let sql = prqlc::compile(
+//!       "from albums | select {title, artist_id}",
+//!        &prqlc::Options::default().no_format()
+//!   )?;
+//!   assert_eq!(&sql[..35], "SELECT title, artist_id FROM albums");
+//!   # Ok(())
+//!   # }
+//!   ```
 //!
 //! - Compile PRQL queries to SQL at build time.
 //!
-//!     For inline strings, use the `prql-compiler-macros` crate; for example:
-//!     ```ignore
-//!     let sql: &str = prql_to_sql!("from albums | select {title, artist_id}");
-//!     ```
+//!   For inline strings, use the `prqlc-macros` crate; for example:
+//!   ```ignore
+//!   let sql: &str = prql_to_sql!("from albums | select {title, artist_id}");
+//!   ```
 //!
-//!     For compiling whole files (`.prql` to `.sql`), call `prqlc`
-//!     from `build.rs`.
-//!     See [this example project](https://github.com/PRQL/prql/tree/main/prqlc/prqlc/examples/compile-files).
+//!   For compiling whole files (`.prql` to `.sql`), call `prqlc` from
+//!   `build.rs`. See [this example
+//!   project](https://github.com/PRQL/prql/tree/main/prqlc/prqlc/examples/compile-files).
 //!
 //! - Compile, format & debug PRQL from command line.
 //!
-//!     ```sh
-//!     $ cargo install --locked prqlc
-//!     $ prqlc compile query.prql
-//!     ```
+//!   ```sh
+//!   $ cargo install --locked prqlc
+//!   $ prqlc compile query.prql
+//!   ```
 //!
 //! ## Feature flags
 //!
-//! The following [feature flags](https://doc.rust-lang.org/cargo/reference/manifest.html#the-features-section) are available:
+//! The following feature flags are available:
 //!
-//! * `serde_yaml`: adapts the `Serialize` implementation for [`prqlc_ast::expr::ExprKind::Literal`]
-//!   to `serde_yaml`, which doesn't support the serialization of nested enums
+//! * `cli`: enables the `prqlc` CLI binary. This is enabled by default. When
+//!   consuming this crate from another rust library, it can be disabled.
+//! * `test-dbs`: enables the `prqlc` in-process test databases as part of the
+//!   crate's tests. This significantly increases compile times so is not
+//!   enabled by default.
+//! * `test-dbs-external`: enables the `prqlc` external test databases,
+//!   requiring a docker container with the test databases to be running. Check
+//!   out the [integration tests](https://github.com/PRQL/prql/tree/main/prqlc/prqlc/tests/integration/dbs)
+//!   for more details.
+//! * `serde_yaml`: Enables serialization and deserialization of ASTs to YAML.
 //!
 //! ## Large binary sizes
 //!
@@ -91,25 +99,33 @@
 // yak-shaving exercise in the future.
 #![allow(clippy::result_large_err)]
 
-mod codegen;
-mod error_message;
-pub mod ir;
-mod parser;
-pub mod semantic;
-pub mod sql;
-mod utils;
-
-pub use error_message::{downcast, ErrorMessage, ErrorMessages, SourceLocation, WithErrorInfo};
-pub use ir::Span;
-pub use prqlc_ast as ast;
-pub use prqlc_ast::error::{Error, Errors, MessageKind, Reason};
+use std::{collections::HashMap, path::PathBuf, str::FromStr};
 
+use anstream::adapter::strip_str;
 use once_cell::sync::Lazy;
 use semver::Version;
 use serde::{Deserialize, Serialize};
-use std::{collections::HashMap, path::PathBuf, str::FromStr};
 use strum::VariantNames;
-use utils::IdGenerator;
+
+pub use error_message::{ErrorMessage, ErrorMessages, SourceLocation};
+pub use prqlc_parser::error::{Error, ErrorSource, Errors, MessageKind, Reason, WithErrorInfo};
+pub use prqlc_parser::lexer::lr;
+pub use prqlc_parser::parser::pr;
+pub use prqlc_parser::span::Span;
+
+mod codegen;
+pub mod debug;
+mod error_message;
+pub mod ir;
+pub mod parser;
+pub mod semantic;
+pub mod sql;
+#[cfg(feature = "cli")]
+pub mod utils;
+#[cfg(not(feature = "cli"))]
+pub(crate) mod utils;
+
+pub type Result<T, E = Error> = core::result::Result<T, E>;
 
 pub static COMPILER_VERSION: Lazy<Version> =
     Lazy::new(|| Version::parse(env!("CARGO_PKG_VERSION")).expect("Invalid prqlc version number"));
@@ -118,7 +134,8 @@ pub static COMPILER_VERSION: Lazy<Version> =
 ///
 /// This is a wrapper for:
 /// - [prql_to_pl] — Build PL AST from a PRQL string
-/// - [pl_to_rq] — Finds variable references, validates functions calls, determines frames and converts PL to RQ.
+/// - [pl_to_rq] — Finds variable references, validates functions calls,
+///   determines frames and converts PL to RQ.
 /// - [rq_to_sql] — Convert RQ AST into an SQL string.
 /// # Example
 /// Use the prql compiler to convert a PRQL string to SQLite dialect
@@ -127,27 +144,43 @@ pub static COMPILER_VERSION: Lazy<Version> =
 /// use prqlc::{compile, Options, Target, sql::Dialect};
 ///
 /// let prql = "from employees | select {name,age}";
-/// let opts = Options {
-///     format: false,
-///     target: Target::Sql(Some(Dialect::SQLite)),
-///     signature_comment: false,
-///     color: false,
-/// };
+/// let opts = Options::default().with_target(Target::Sql(Some(Dialect::SQLite))).with_signature_comment(false).with_format(false);
 /// let sql = compile(&prql, &opts).unwrap();
 /// println!("PRQL: {}\nSQLite: {}", prql, &sql);
 /// assert_eq!("SELECT name, age FROM employees", sql)
 ///
 /// ```
-/// See [`sql::Options`](sql/struct.Options.html) and [`sql::Dialect`](sql/enum.Dialect.html) for options and supported SQL dialects.
+/// See [`sql::Options`](sql/struct.Options.html) and
+/// [`sql::Dialect`](sql/enum.Dialect.html) for options and supported SQL
+/// dialects.
 pub fn compile(prql: &str, options: &Options) -> Result<String, ErrorMessages> {
-    let mut sources = SourceTree::from(prql);
-    semantic::load_std_lib(&mut sources);
+    let sources = SourceTree::from(prql);
 
-    parser::parse(&sources)
-        .and_then(|ast| semantic::resolve_and_lower(ast, &[], None))
-        .and_then(|rq| sql::compile(rq, options))
-        .map_err(error_message::downcast)
-        .map_err(|e| e.composed(&prql.into()))
+    Ok(&sources)
+        .and_then(parser::parse)
+        .and_then(|ast| {
+            semantic::resolve_and_lower(ast, &[], None)
+                .map_err(|e| e.with_source(ErrorSource::NameResolver).into())
+        })
+        .and_then(|rq| {
+            sql::compile(rq, options).map_err(|e| e.with_source(ErrorSource::SQL).into())
+        })
+        .map_err(|e| {
+            let error_messages = ErrorMessages::from(e).composed(&sources);
+            match options.display {
+                DisplayOptions::AnsiColor => error_messages,
+                DisplayOptions::Plain => ErrorMessages {
+                    inner: error_messages
+                        .inner
+                        .into_iter()
+                        .map(|e| ErrorMessage {
+                            display: e.display.map(|s| strip_str(&s).to_string()),
+                            ..e
+                        })
+                        .collect(),
+                },
+            }
+        })
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
@@ -211,16 +244,21 @@ pub struct Options {
     /// Defaults to true.
     pub signature_comment: bool,
 
-    /// Whether to use ANSI colors in error messages. This is deprecated and has
-    /// no effect.
+    /// Deprecated: use `display` instead.
+    pub color: bool,
+
+    /// Whether to use ANSI colors in error messages. This may be extended to
+    /// other formats in the future.
     ///
-    /// Instead, in order of preference:
-    /// - Use a library such as `anstream` to encapsulate the presentation
-    ///   logic.
+    /// Note that we don't generally recommend threading a `color` option
+    /// through an entire application. Instead, in order of preferences:
+    /// - Use a library such as `anstream` to encapsulate presentation logic and
+    ///   automatically disable colors when not connected to a TTY.
     /// - Set an environment variable such as `CLI_COLOR=0` to disable any
     ///   colors coming back from this library.
-    /// - Strip colors from the output (possibly also with a library such as `anstream`)
-    pub color: bool,
+    /// - Strip colors from the output (possibly also with a library such as
+    ///   `anstream`).
+    pub display: DisplayOptions,
 }
 
 impl Default for Options {
@@ -229,7 +267,8 @@ impl Default for Options {
             format: true,
             target: Target::Sql(None),
             signature_comment: true,
-            color: false,
+            color: true,
+            display: DisplayOptions::AnsiColor,
         }
     }
 }
@@ -258,62 +297,79 @@ impl Options {
         self
     }
 
-    #[deprecated(note = "`color` now has no effect; see `Options` docs for more details")]
+    #[deprecated(note = "`color` is replaced by `display`; see `Options` docs for more details")]
     pub fn with_color(mut self, color: bool) -> Self {
         self.color = color;
         self
     }
+
+    pub fn with_display(mut self, display: DisplayOptions) -> Self {
+        self.display = display;
+        self
+    }
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize, strum::EnumString)]
+#[strum(serialize_all = "snake_case")]
+#[non_exhaustive]
+pub enum DisplayOptions {
+    /// Plain text
+    Plain,
+    /// With ANSI colors
+    AnsiColor,
 }
 
 #[doc = include_str!("../README.md")]
 #[cfg(doctest)]
 pub struct ReadmeDoctests;
 
-/// Parse PRQL into a PL AST
-// TODO: rename this to `prql_to_pl_simple`
-pub fn prql_to_pl(prql: &str) -> Result<Vec<prqlc_ast::stmt::Stmt>, ErrorMessages> {
-    let sources = SourceTree::from(prql);
-
-    parser::parse(&sources)
-        .map(|x| x.sources.into_values().next().unwrap())
-        .map_err(error_message::downcast)
-        .map_err(|e| e.composed(&prql.into()))
+/// Lex PRQL source into Lexer Representation.
+pub fn prql_to_tokens(prql: &str) -> Result<lr::Tokens, ErrorMessages> {
+    prqlc_parser::lexer::lex_source(prql).map_err(|e| {
+        e.into_iter()
+            .map(|e| e.into())
+            .collect::<Vec<ErrorMessage>>()
+            .into()
+    })
 }
 
 /// Parse PRQL into a PL AST
-pub fn prql_to_pl_tree(
-    prql: &SourceTree,
-) -> Result<SourceTree<Vec<prqlc_ast::stmt::Stmt>>, ErrorMessages> {
-    parser::parse(prql)
-        .map_err(error_message::downcast)
-        .map_err(|e| e.composed(prql))
+// TODO: rename this to `prql_to_pl_simple`
+pub fn prql_to_pl(prql: &str) -> Result<pr::ModuleDef, ErrorMessages> {
+    let source_tree = SourceTree::from(prql);
+    prql_to_pl_tree(&source_tree)
+}
+
+/// Parse PRQL into a PL AST
+pub fn prql_to_pl_tree(prql: &SourceTree) -> Result<pr::ModuleDef, ErrorMessages> {
+    parser::parse(prql).map_err(|e| ErrorMessages::from(e).composed(prql))
 }
 
 /// Perform semantic analysis and convert PL to RQ.
 // TODO: rename this to `pl_to_rq_simple`
-pub fn pl_to_rq(pl: Vec<prqlc_ast::stmt::Stmt>) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
-    let source_tree = SourceTree::single(PathBuf::new(), pl);
-    semantic::resolve_and_lower(source_tree, &[], None).map_err(error_message::downcast)
+pub fn pl_to_rq(pl: pr::ModuleDef) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
+    semantic::resolve_and_lower(pl, &[], None)
+        .map_err(|e| e.with_source(ErrorSource::NameResolver).into())
 }
 
 /// Perform semantic analysis and convert PL to RQ.
 pub fn pl_to_rq_tree(
-    pl: SourceTree<Vec<prqlc_ast::stmt::Stmt>>,
+    pl: pr::ModuleDef,
     main_path: &[String],
     database_module_path: &[String],
 ) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
     semantic::resolve_and_lower(pl, main_path, Some(database_module_path))
-        .map_err(error_message::downcast)
+        .map_err(|e| e.with_source(ErrorSource::NameResolver).into())
 }
 
 /// Generate SQL from RQ.
 pub fn rq_to_sql(rq: ir::rq::RelationalQuery, options: &Options) -> Result<String, ErrorMessages> {
-    sql::compile(rq, options).map_err(error_message::downcast)
+    sql::compile(rq, options).map_err(|e| e.with_source(ErrorSource::SQL).into())
 }
 
 /// Generate PRQL code from PL AST
-pub fn pl_to_prql(pl: Vec<prqlc_ast::stmt::Stmt>) -> Result<String, ErrorMessages> {
-    Ok(codegen::WriteSource::write(&pl, codegen::WriteOpt::default()).unwrap())
+pub fn pl_to_prql(pl: &pr::ModuleDef) -> Result<String, ErrorMessages> {
+    Ok(codegen::WriteSource::write(&pl.stmts, codegen::WriteOpt::default()).unwrap())
 }
 
 /// JSON serialization and deserialization functions
@@ -321,23 +377,27 @@ pub mod json {
     use super::*;
 
     /// JSON serialization
-    pub fn from_pl(pl: Vec<prqlc_ast::stmt::Stmt>) -> Result<String, ErrorMessages> {
-        serde_json::to_string(&pl).map_err(|e| anyhow::anyhow!(e).into())
+    pub fn from_pl(pl: &pr::ModuleDef) -> Result<String, ErrorMessages> {
+        serde_json::to_string(pl).map_err(convert_json_err)
     }
 
     /// JSON deserialization
-    pub fn to_pl(json: &str) -> Result<Vec<prqlc_ast::stmt::Stmt>, ErrorMessages> {
-        serde_json::from_str(json).map_err(|e| anyhow::anyhow!(e).into())
+    pub fn to_pl(json: &str) -> Result<pr::ModuleDef, ErrorMessages> {
+        serde_json::from_str(json).map_err(convert_json_err)
     }
 
     /// JSON serialization
-    pub fn from_rq(rq: ir::rq::RelationalQuery) -> Result<String, ErrorMessages> {
-        serde_json::to_string(&rq).map_err(|e| anyhow::anyhow!(e).into())
+    pub fn from_rq(rq: &ir::rq::RelationalQuery) -> Result<String, ErrorMessages> {
+        serde_json::to_string(rq).map_err(convert_json_err)
     }
 
     /// JSON deserialization
     pub fn to_rq(json: &str) -> Result<ir::rq::RelationalQuery, ErrorMessages> {
-        serde_json::from_str(json).map_err(|e| anyhow::anyhow!(e).into())
+        serde_json::from_str(json).map_err(convert_json_err)
+    }
+
+    fn convert_json_err(err: serde_json::Error) -> ErrorMessages {
+        ErrorMessages::from(Error::new_simple(err.to_string()))
     }
 }
 
@@ -348,20 +408,20 @@ pub mod json {
 // (i.e. `Option<PathBuf>` below signifies whether it's a project or not). But
 // waiting until it's necessary before splitting it out.)
 #[derive(Debug, Clone, Default, Serialize)]
-pub struct SourceTree<T: Sized + Serialize = String> {
+pub struct SourceTree {
     /// Path to the root of the source tree.
     pub root: Option<PathBuf>,
 
     /// Mapping from file paths into into their contents.
     /// Paths are relative to the root.
-    pub sources: HashMap<PathBuf, T>,
+    pub sources: HashMap<PathBuf, String>,
 
     /// Index of source ids to paths. Used to keep [error::Span] lean.
     source_ids: HashMap<u16, PathBuf>,
 }
 
-impl<T: Sized + Serialize> SourceTree<T> {
-    pub fn single(path: PathBuf, content: T) -> Self {
+impl SourceTree {
+    pub fn single(path: PathBuf, content: String) -> Self {
         SourceTree {
             sources: [(path.clone(), content)].into(),
             source_ids: [(1, path)].into(),
@@ -371,43 +431,27 @@ impl<T: Sized + Serialize> SourceTree<T> {
 
     pub fn new<I>(iter: I, root: Option<PathBuf>) -> Self
     where
-        I: IntoIterator<Item = (PathBuf, T)>,
+        I: IntoIterator<Item = (PathBuf, String)>,
     {
-        let mut id_gen = IdGenerator::<usize>::new();
         let mut res = SourceTree {
             sources: HashMap::new(),
             source_ids: HashMap::new(),
             root,
         };
 
-        for (path, content) in iter {
+        for (index, (path, content)) in iter.into_iter().enumerate() {
             res.sources.insert(path.clone(), content);
-            res.source_ids.insert(id_gen.gen() as u16, path);
+            res.source_ids.insert((index + 1) as u16, path);
         }
         res
     }
 
-    pub fn insert(&mut self, path: PathBuf, content: T) {
+    pub fn insert(&mut self, path: PathBuf, content: String) {
         let last_id = self.source_ids.keys().max().cloned().unwrap_or(0);
         self.sources.insert(path.clone(), content);
         self.source_ids.insert(last_id + 1, path);
     }
 
-    pub fn map<F, U: Sized + Serialize>(self, f: F) -> SourceTree<U>
-    where
-        F: Fn(T) -> U,
-    {
-        SourceTree {
-            sources: self
-                .sources
-                .into_iter()
-                .map(|(path, val)| (path, f(val)))
-                .collect(),
-            source_ids: self.source_ids,
-            root: self.root,
-        }
-    }
-
     pub fn get_path(&self, source_id: u16) -> Option<&PathBuf> {
         self.source_ids.get(&source_id)
     }
@@ -415,17 +459,55 @@ impl<T: Sized + Serialize> SourceTree<T> {
 
 impl<S: ToString> From<S> for SourceTree {
     fn from(source: S) -> Self {
-        SourceTree::single(std::path::Path::new("").to_path_buf(), source.to_string())
+        SourceTree::single(PathBuf::from(""), source.to_string())
+    }
+}
+
+/// Debugging and unstable API functions
+pub mod internal {
+    use super::*;
+
+    /// Create column-level lineage graph
+    pub fn pl_to_lineage(
+        pl: pr::ModuleDef,
+    ) -> Result<semantic::reporting::FrameCollector, ErrorMessages> {
+        let ast = Some(pl.clone());
+
+        let root_module = semantic::resolve(pl).map_err(ErrorMessages::from)?;
+
+        let (main, _) = root_module.find_main_rel(&[]).unwrap();
+        let mut fc =
+            semantic::reporting::collect_frames(*main.clone().into_relation_var().unwrap());
+        fc.ast = ast;
+
+        Ok(fc)
+    }
+
+    pub mod json {
+        use super::*;
+
+        /// JSON serialization of FrameCollector lineage
+        pub fn from_lineage(
+            fc: &semantic::reporting::FrameCollector,
+        ) -> Result<String, ErrorMessages> {
+            serde_json::to_string(fc).map_err(convert_json_err)
+        }
+
+        fn convert_json_err(err: serde_json::Error) -> ErrorMessages {
+            ErrorMessages::from(Error::new_simple(err.to_string()))
+        }
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::Target;
-    use insta::assert_debug_snapshot;
-    use prqlc_ast::expr::Ident;
     use std::str::FromStr;
 
+    use insta::assert_debug_snapshot;
+
+    use crate::pr::Ident;
+    use crate::Target;
+
     pub fn compile(prql: &str) -> Result<String, super::ErrorMessages> {
         anstream::ColorChoice::Never.write_global();
         super::compile(prql, &super::Options::default().no_signature())
