@@ -7,7 +7,7 @@
 //! The name "savvy" comes from the Japanese word "錆" (pronounced as `sàbí`), which means "Rust".
 //!
 //! With savvy, you can automatically generate R functions from Rust code. This is
-//! an example of what savvy-powered function would look like.
+//! an example of what a savvy-powered function would look like.
 //!
 //! **Rust**:
 //!
@@ -70,6 +70,7 @@ pub mod log;
 use std::os::raw::c_char;
 
 pub use error::{Error, Result};
+pub use sexp::environment::EnvironmentSexp;
 pub use sexp::external_pointer::{
     get_external_pointer_addr, take_external_pointer_value, ExternalPointerSexp, IntoExtPtrSexp,
 };
@@ -80,6 +81,7 @@ pub use sexp::logical::{LogicalSexp, OwnedLogicalSexp};
 pub use sexp::na::NotAvailableValue;
 pub use sexp::null::NullSexp;
 pub use sexp::numeric::{NumericScalar, NumericSexp, NumericTypedSexp};
+pub use sexp::raw::{OwnedRawSexp, RawSexp};
 pub use sexp::real::{OwnedRealSexp, RealSexp};
 pub use sexp::string::{OwnedStringSexp, StringSexp};
 pub use sexp::{Sexp, TypedSexp};
