@@ -2,22 +2,23 @@ use std::collections::hash_map::RandomState;
 use std::collections::{HashMap, HashSet};
 use std::iter::zip;
 
-use anyhow::Result;
 use enum_as_inner::EnumAsInner;
 use itertools::Itertools;
-use prqlc_ast::TupleField;
+use prqlc_parser::generic::{InterpolateItem, Range, SwitchCase};
+use prqlc_parser::lexer::lr::Literal;
 
 use crate::ir::decl::{self, DeclKind, Module, RootModule, TableExpr};
 use crate::ir::generic::{ColumnSort, WindowFrame};
+use crate::ir::pl::TableExternRef::LocalTable;
 use crate::ir::pl::{self, Ident, Lineage, LineageColumn, PlFold, QueryDef};
 use crate::ir::rq::{
     self, CId, RelationColumn, RelationLiteral, RelationalQuery, TId, TableDecl, Transform,
 };
+use crate::pr::TyTupleField;
 use crate::semantic::write_pl;
 use crate::utils::{toposort, IdGenerator};
 use crate::COMPILER_VERSION;
-use crate::{Error, Reason, Span, WithErrorInfo};
-use prqlc_ast::expr::generic::{InterpolateItem, Range, SwitchCase};
+use crate::{Error, Reason, Result, Span, WithErrorInfo};
 
 /// Convert a resolved expression at path `main_path` relative to `root_mod`
 /// into RQ and make sure that:
@@ -80,7 +81,7 @@ pub fn lower_to_ir(
 }
 
 fn extern_ref_to_relation(
-    mut columns: Vec<TupleField>,
+    mut columns: Vec<TyTupleField>,
     fq_ident: &Ident,
     database_module_path: &[String],
 ) -> Result<(rq::Relation, Option<String>), Error> {
@@ -103,21 +104,21 @@ fn extern_ref_to_relation(
     };
 
     // put wildcards last
-    columns.sort_by_key(|a| matches!(a, TupleField::Wildcard(_)));
+    columns.sort_by_key(|a| matches!(a, TyTupleField::Wildcard(_)));
 
     let relation = rq::Relation {
-        kind: rq::RelationKind::ExternRef(extern_name),
+        kind: rq::RelationKind::ExternRef(LocalTable(extern_name)),
         columns: tuple_fields_to_relation_columns(columns),
     };
     Ok((relation, None))
 }
 
-fn tuple_fields_to_relation_columns(columns: Vec<TupleField>) -> Vec<RelationColumn> {
+fn tuple_fields_to_relation_columns(columns: Vec<TyTupleField>) -> Vec<RelationColumn> {
     columns
         .into_iter()
         .map(|field| match field {
-            TupleField::Single(name, _) => RelationColumn::Single(name),
-            TupleField::Wildcard(_) => RelationColumn::Wildcard,
+            TyTupleField::Single(name, _) => RelationColumn::Single(name),
+            TyTupleField::Wildcard(_) => RelationColumn::Wildcard,
         })
         .collect_vec()
 }
@@ -125,7 +126,7 @@ fn tuple_fields_to_relation_columns(columns: Vec<TupleField>) -> Vec<RelationCol
 fn validate_query_def(query_def: &QueryDef) -> Result<()> {
     if let Some(requirement) = &query_def.version {
         if !requirement.matches(&COMPILER_VERSION) {
-            return Err(Error::new_simple("This query uses a version of PRQL that is not supported by prqlc. Please upgrade the compiler.").into());
+            return Err(Error::new_simple("This query uses a version of PRQL that is not supported by prqlc. Please upgrade the compiler."));
         }
     }
     Ok(())
@@ -142,7 +143,7 @@ struct Lowerer {
     /// describes what has certain id has been lowered to
     node_mapping: HashMap<usize, LoweredTarget>,
 
-    /// mapping from [Ident] of [crate::ast::TableDef] into [TId]s
+    /// mapping from [Ident] of [crate::pr::TableDef] into [TId]s
     table_mapping: HashMap<Ident, TId>,
 
     // current window for any new column defs
@@ -225,7 +226,10 @@ impl Lowerer {
             pl::ExprKind::Ident(fq_table_name) => {
                 // ident that refer to table: create an instance of the table
                 let id = expr.id.unwrap();
-                let tid = *self.table_mapping.get(&fq_table_name).unwrap();
+                let tid = *self
+                    .table_mapping
+                    .get(&fq_table_name)
+                    .ok_or_else(|| Error::new_bug(4474))?;
 
                 log::debug!("lowering an instance of table {fq_table_name} (id={id})...");
 
@@ -273,7 +277,7 @@ impl Lowerer {
 
                 // pull columns from the table decl
                 let frame = expr.lineage.as_ref().unwrap();
-                let input = frame.inputs.get(0).unwrap();
+                let input = frame.inputs.first().unwrap();
 
                 let table_decl = self.root_mod.module.get(&input.table).unwrap();
                 let table_decl = table_decl.kind.as_table_decl().unwrap();
@@ -307,7 +311,7 @@ impl Lowerer {
 
                 // pull columns from the table decl
                 let frame = expr.lineage.as_ref().unwrap();
-                let input = frame.inputs.get(0).unwrap();
+                let input = frame.inputs.first().unwrap();
 
                 let table_decl = self.root_mod.module.get(&input.table).unwrap();
                 let table_decl = table_decl.kind.as_table_decl().unwrap();
@@ -397,8 +401,7 @@ impl Lowerer {
                     found: format!("`{}`", write_pl(expr.clone())),
                 })
                 .push_hint("are you missing `from` statement?")
-                .with_span(expr.span)
-                .into())
+                .with_span(expr.span))
             }
         })
     }
@@ -741,8 +744,7 @@ impl Lowerer {
                         who: None,
                         expected: "an identifier".to_string(),
                         found: write_pl(e),
-                    })
-                    .into());
+                    }));
                 }
             }
         }
@@ -832,8 +834,7 @@ impl Lowerer {
                     } else {
                         return Err(
                             Error::new_simple("This wildcard usage is not yet supported.")
-                                .with_span(span)
-                                .into(),
+                                .with_span(span),
                         );
                     }
                 } else if let Some(id) = expr.target_id {
@@ -855,8 +856,7 @@ impl Lowerer {
                 } else {
                     return Err(
                         Error::new_simple("This wildcard usage is not yet supported.")
-                            .with_span(span)
-                            .into(),
+                            .with_span(span),
                     );
                 }
             }
@@ -900,8 +900,7 @@ impl Lowerer {
                 return Err(
                     Error::new_simple("table instance cannot be referenced directly")
                         .push_hint("did you forget to specify the column name?")
-                        .with_span(span)
-                        .into(),
+                        .with_span(span),
                 );
             }
 
@@ -918,12 +917,14 @@ impl Lowerer {
                     found: format!("`{}`", write_pl(expr.clone())),
                 })
                 .push_hint("this is probably a 'bad type' error (we are working on that)")
-                .with_span(expr.span)
-                .into());
+                .with_span(expr.span));
             }
 
             pl::ExprKind::Internal(_) => {
-                panic!("Unresolved lowering: {}", write_pl(expr))
+                return Err(Error::new_assert(format!(
+                    "Unresolved lowering: {}",
+                    write_pl(expr)
+                )))
             }
         };
 
@@ -958,8 +959,7 @@ impl Lowerer {
                         "This table contains unnamed columns that need to be referenced by name",
                     )
                     .with_span(self.root_mod.span_map.get(&id).cloned())
-                    .push_hint("the name may have been overridden later in the pipeline.")
-                    .into()),
+                    .push_hint("the name may have been overridden later in the pipeline.")),
                 };
                 log::trace!("lookup cid of name={name:?} in input {input_columns:?}");
 
@@ -970,7 +970,7 @@ impl Lowerer {
                 }
             }
             None => {
-                return Err(Error::new(Reason::Bug { issue: Some(3870) }))?;
+                return Err(Error::new_bug(3870))?;
             }
         };
 
@@ -980,7 +980,7 @@ impl Lowerer {
 
 fn str_lit(string: String) -> rq::Expr {
     rq::Expr {
-        kind: rq::ExprKind::Literal(pl::Literal::String(string)),
+        kind: rq::ExprKind::Literal(Literal::String(string)),
         span: None,
     }
 }
@@ -1020,8 +1020,7 @@ fn validate_take_range(range: &Range<rq::Expr>, span: Option<Span>) -> Result<()
             expected: "a positive int range".to_string(),
             found: range_display,
         })
-        .with_span(span)
-        .into())
+        .with_span(span))
     } else {
         Ok(())
     }
@@ -1134,20 +1133,15 @@ fn get_span_of_id(l: &Lowerer, id: Option<usize>) -> Option<Span> {
     id.and_then(|id| l.root_mod.span_map.get(&id)).cloned()
 }
 
-fn with_span_if_not_exists<'a, F>(get_span: F) -> impl FnOnce(anyhow::Error) -> anyhow::Error + 'a
+fn with_span_if_not_exists<'a, F>(get_span: F) -> impl FnOnce(Error) -> Error + 'a
 where
     F: FnOnce() -> Option<Span> + 'a,
 {
     move |e| {
-        let e = match e.downcast::<Error>() {
-            Ok(e) => e,
-            Err(e) => return e,
-        };
-
         if e.span.is_some() {
-            return e.into();
+            return e;
         }
 
-        e.with_span(get_span()).into()
+        e.with_span(get_span())
     }
 }
