@@ -1,18 +1,21 @@
 use std::ffi::{CStr, CString};
 
 use savvy_ffi::{
-    R_NilValue, Rf_getAttrib, Rf_isFunction, Rf_isInteger, Rf_isLogical, Rf_isNumeric, Rf_isReal,
-    Rf_isString, Rf_type2char, Rf_xlength, EXTPTRSXP, INTEGER, SEXP, SEXPTYPE, TYPEOF, VECSXP,
+    R_NilValue, Rboolean_TRUE, Rf_getAttrib, Rf_isEnvironment, Rf_isFunction, Rf_isInteger,
+    Rf_isLogical, Rf_isNumeric, Rf_isReal, Rf_isString, Rf_type2char, Rf_xlength, EXTPTRSXP,
+    INTEGER, RAWSXP, SEXP, SEXPTYPE, TYPEOF, VECSXP,
 };
 
 use crate::{
-    ExternalPointerSexp, IntegerSexp, ListSexp, LogicalSexp, NullSexp, OwnedIntegerSexp,
-    OwnedLogicalSexp, OwnedRealSexp, OwnedStringSexp, RealSexp, StringSexp,
+    EnvironmentSexp, ExternalPointerSexp, FunctionSexp, IntegerSexp, ListSexp, LogicalSexp,
+    NullSexp, OwnedIntegerSexp, OwnedLogicalSexp, OwnedRawSexp, OwnedRealSexp, OwnedStringSexp,
+    RawSexp, RealSexp, StringSexp,
 };
 
 #[cfg(feature = "complex")]
 use crate::{ComplexSexp, OwnedComplexSexp};
 
+pub mod environment;
 pub mod external_pointer;
 pub mod function;
 pub mod integer;
@@ -20,6 +23,7 @@ pub mod list;
 pub mod logical;
 pub mod na;
 pub mod null;
+pub mod raw;
 pub mod real;
 pub mod scalar;
 pub mod string;
@@ -42,28 +46,34 @@ impl Sexp {
 
     /// Returns `true` if the SEXP is an integer vector.
     pub fn is_integer(&self) -> bool {
-        unsafe { Rf_isInteger(self.0) == 1 }
+        unsafe { Rf_isInteger(self.0) == Rboolean_TRUE }
     }
 
     /// Returns `true` if the SEXP is a real vector.
     pub fn is_real(&self) -> bool {
-        unsafe { Rf_isReal(self.0) == 1 }
+        unsafe { Rf_isReal(self.0) == Rboolean_TRUE }
     }
 
     /// Returns `true` if the SEXP is a real or integer vector.
     pub fn is_numeric(&self) -> bool {
-        unsafe { Rf_isNumeric(self.0) == 1 }
+        unsafe { Rf_isNumeric(self.0) == Rboolean_TRUE }
     }
 
     #[cfg(feature = "complex")]
     /// Returns `true` if the SEXP is a complex.
     pub fn is_complex(&self) -> bool {
-        unsafe { savvy_ffi::Rf_isComplex(self.0) == 1 }
+        unsafe { savvy_ffi::Rf_isComplex(self.0) == Rboolean_TRUE }
     }
 
     /// Returns `true` if the SEXP is a logical vector.
     pub fn is_logical(&self) -> bool {
-        unsafe { Rf_isLogical(self.0) == 1 }
+        unsafe { Rf_isLogical(self.0) == Rboolean_TRUE }
+    }
+
+    /// Returns `true` if the SEXP is a raw vector.
+    pub fn is_raw(&self) -> bool {
+        // There's no test function for RAWSXP
+        unsafe { TYPEOF(self.0) as u32 == RAWSXP }
     }
 
     /// Returns `true` if the SEXP is a character vector.
@@ -72,7 +82,7 @@ impl Sexp {
         //
         // macro version: https://github.com/wch/r-source/blob/9065779ee510b7bd8ca93d08f4dd4b6e2bd31923/src/include/Defn.h#L759
         // function version: https://github.com/wch/r-source/blob/9065779ee510b7bd8ca93d08f4dd4b6e2bd31923/src/main/memory.c#L4460
-        unsafe { Rf_isString(self.0) == 1 }
+        unsafe { Rf_isString(self.0) == Rboolean_TRUE }
     }
 
     /// Returns `true` if the SEXP is a list.
@@ -88,7 +98,11 @@ impl Sexp {
 
     /// Returns `true` if the SEXP is a function.
     pub fn is_function(&self) -> bool {
-        unsafe { Rf_isFunction(self.0) == 1 }
+        unsafe { Rf_isFunction(self.0) == Rboolean_TRUE }
+    }
+
+    pub fn is_environment(&self) -> bool {
+        unsafe { Rf_isEnvironment(self.0) == Rboolean_TRUE }
     }
 
     fn is_sexp_type(&self, sexptype: SEXPTYPE) -> bool {
@@ -98,11 +112,13 @@ impl Sexp {
             #[cfg(feature = "complex")]
             savvy_ffi::CPLXSXP => self.is_complex(),
             savvy_ffi::LGLSXP => self.is_logical(),
+            savvy_ffi::RAWSXP => self.is_raw(),
             savvy_ffi::STRSXP => self.is_string(),
             savvy_ffi::VECSXP => self.is_list(),
             savvy_ffi::EXTPTRSXP => self.is_external_pointer(),
             // cf. https://github.com/wch/r-source/blob/95ac44a87065d5b42579b621d278adc44641dcf0/src/include/Rinlinedfuns.h#L810-L815
             savvy_ffi::CLOSXP | savvy_ffi::BUILTINSXP | savvy_ffi::SPECIALSXP => self.is_function(),
+            savvy_ffi::ENVSXP => self.is_environment(),
             savvy_ffi::NILSXP => self.is_null(),
             _ => false,
         }
@@ -128,10 +144,10 @@ macro_rules! impl_sexp_type_assert {
         if $self.is_sexp_type(savvy_ffi::$sexptype) {
             Ok(())
         } else {
-            let expected = unsafe { get_human_readable_type_name(savvy_ffi::$sexptype) };
-            let actual = $self.get_human_readable_type_name();
-            let msg = format!("expected: {expected}\n  actual: {actual}");
-            Err(crate::error::Error::UnexpectedType(msg))
+            let expected =
+                unsafe { get_human_readable_type_name(savvy_ffi::$sexptype).to_string() };
+            let actual = $self.get_human_readable_type_name().to_string();
+            Err(crate::error::Error::UnexpectedType { expected, actual })
         }
     };
 }
@@ -163,6 +179,11 @@ impl Sexp {
         impl_sexp_type_assert!(self, LGLSXP)
     }
 
+    /// Returns error when the SEXP is not a raw vector.
+    pub fn assert_raw(&self) -> crate::error::Result<()> {
+        impl_sexp_type_assert!(self, RAWSXP)
+    }
+
     /// Returns error when the SEXP is not a string vector.
     pub fn assert_string(&self) -> crate::error::Result<()> {
         impl_sexp_type_assert!(self, STRSXP)
@@ -182,6 +203,11 @@ impl Sexp {
     pub fn assert_function(&self) -> crate::error::Result<()> {
         impl_sexp_type_assert!(self, CLOSXP)
     }
+
+    /// Returns error when the SEXP is not a function.
+    pub fn assert_environment(&self) -> crate::error::Result<()> {
+        impl_sexp_type_assert!(self, ENVSXP)
+    }
 }
 
 #[non_exhaustive]
@@ -192,11 +218,13 @@ pub enum TypedSexp {
     #[cfg(feature = "complex")]
     Complex(ComplexSexp),
     Logical(LogicalSexp),
+    Raw(RawSexp),
     String(StringSexp),
     List(ListSexp),
     Null(NullSexp),
     ExternalPointer(ExternalPointerSexp),
     Function(FunctionSexp),
+    Environment(EnvironmentSexp),
     Other(SEXP),
 }
 
@@ -215,10 +243,12 @@ into_typed_sxp!(RealSexp, Real);
 #[cfg(feature = "complex")]
 into_typed_sxp!(ComplexSexp, Complex);
 into_typed_sxp!(LogicalSexp, Logical);
+into_typed_sxp!(RawSexp, Raw);
 into_typed_sxp!(StringSexp, String);
 into_typed_sxp!(ListSexp, List);
 into_typed_sxp!(ExternalPointerSexp, ExternalPointer);
 into_typed_sxp!(FunctionSexp, Function);
+into_typed_sxp!(EnvironmentSexp, Environment);
 into_typed_sxp!(NullSexp, Null);
 
 macro_rules! into_typed_sxp_owned {
@@ -235,8 +265,9 @@ into_typed_sxp_owned!(OwnedIntegerSexp, Integer);
 into_typed_sxp_owned!(OwnedRealSexp, Real);
 #[cfg(feature = "complex")]
 into_typed_sxp_owned!(OwnedComplexSexp, Complex);
-into_typed_sxp_owned!(OwnedStringSexp, String);
 into_typed_sxp_owned!(OwnedLogicalSexp, Logical);
+into_typed_sxp_owned!(OwnedRawSexp, Raw);
+into_typed_sxp_owned!(OwnedStringSexp, String);
 
 impl From<TypedSexp> for SEXP {
     fn from(value: TypedSexp) -> Self {
@@ -246,11 +277,13 @@ impl From<TypedSexp> for SEXP {
             TypedSexp::Real(sxp) => sxp.inner(),
             #[cfg(feature = "complex")]
             TypedSexp::Complex(sxp) => sxp.inner(),
-            TypedSexp::String(sxp) => sxp.inner(),
             TypedSexp::Logical(sxp) => sxp.inner(),
+            TypedSexp::Raw(sxp) => sxp.inner(),
+            TypedSexp::String(sxp) => sxp.inner(),
             TypedSexp::List(sxp) => sxp.inner(),
             TypedSexp::ExternalPointer(sxp) => sxp.inner(),
             TypedSexp::Function(sxp) => sxp.inner(),
+            TypedSexp::Environment(sxp) => sxp.inner(),
             TypedSexp::Other(sxp) => sxp,
         }
     }
@@ -266,6 +299,7 @@ impl Sexp {
             #[cfg(feature = "complex")]
             savvy_ffi::CPLXSXP => TypedSexp::Complex(ComplexSexp(self.0)),
             savvy_ffi::LGLSXP => TypedSexp::Logical(LogicalSexp(self.0)),
+            savvy_ffi::RAWSXP => TypedSexp::Raw(RawSexp(self.0)),
             savvy_ffi::STRSXP => TypedSexp::String(StringSexp(self.0)),
             savvy_ffi::VECSXP => TypedSexp::List(ListSexp(self.0)),
             savvy_ffi::EXTPTRSXP => TypedSexp::ExternalPointer(ExternalPointerSexp(self.0)),
@@ -273,6 +307,7 @@ impl Sexp {
             savvy_ffi::CLOSXP | savvy_ffi::BUILTINSXP | savvy_ffi::SPECIALSXP => {
                 TypedSexp::Function(FunctionSexp(self.0))
             }
+            savvy_ffi::ENVSXP => TypedSexp::Environment(EnvironmentSexp(self.0)),
             savvy_ffi::NILSXP => TypedSexp::Null(NullSexp),
             _ => TypedSexp::Other(self.0),
         }
@@ -555,5 +590,3 @@ macro_rules! impl_common_sexp_ops_owned {
 
 pub(crate) use impl_common_sexp_ops;
 pub(crate) use impl_common_sexp_ops_owned;
-
-use self::function::FunctionSexp;
