@@ -11,14 +11,15 @@
 //!
 //! As a consequence, generated SQL may be verbose, since it will avoid newer or less adopted SQL
 //! constructs. The upside is much less complex translator.
-use anyhow::{bail, Result};
 use core::fmt::Debug;
+use std::any::{Any, TypeId};
 
 use chrono::format::{Fixed, Item, Numeric, Pad, StrftimeItems};
 use serde::{Deserialize, Serialize};
-use std::any::{Any, TypeId};
 use strum::VariantNames;
 
+use crate::{Error, Result};
+
 /// SQL dialect.
 ///
 /// This only changes the output for a relatively small subset of features.
@@ -166,7 +167,7 @@ pub(super) trait DialectHandler: Any + Debug {
     }
 
     /// Whether or not intervals such as `INTERVAL 1 HOUR` require quotes like
-    /// `INTERVAL '1' HOUR`
+    /// `INTERVAL '1 HOUR'`
     fn requires_quotes_intervals(&self) -> bool {
         false
     }
@@ -191,7 +192,13 @@ pub(super) trait DialectHandler: Any + Debug {
     }
 
     fn translate_chrono_item(&self, _item: Item) -> Result<String> {
-        bail!("Date formatting is not yet supported for this dialect")
+        Err(Error::new_simple(
+            "Date formatting is not yet supported for this dialect",
+        ))
+    }
+
+    fn supports_zero_columns(&self) -> bool {
+        false
     }
 }
 
@@ -204,7 +211,7 @@ impl dyn DialectHandler {
 
 impl DialectHandler for GenericDialect {
     fn translate_chrono_item(&self, _item: Item) -> Result<String> {
-        bail!("Date formatting requires a dialect")
+        Err(Error::new_simple("Date formatting requires a dialect"))
     }
 }
 
@@ -251,9 +258,17 @@ impl DialectHandler for PostgresDialect {
                 }
             }
             Item::Space(spaces) => spaces.to_string(),
-            _ => bail!("PRQL doesn't support this format specifier"),
+            _ => {
+                return Err(Error::new_simple(
+                    "PRQL doesn't support this format specifier",
+                ))
+            }
         })
     }
+
+    fn supports_zero_columns(&self) -> bool {
+        true
+    }
 }
 
 impl DialectHandler for GlareDbDialect {
@@ -331,7 +346,11 @@ impl DialectHandler for MsSqlDialect {
                 }
             }
             Item::Space(spaces) => spaces.to_string(),
-            _ => bail!("PRQL doesn't support this format specifier"),
+            _ => {
+                return Err(Error::new_simple(
+                    "PRQL doesn't support this format specifier",
+                ))
+            }
         })
     }
 }
@@ -369,7 +388,11 @@ impl DialectHandler for MySqlDialect {
             Item::Fixed(Fixed::RFC3339) => "%Y-%m-%dT%H:%i:%S.%fZ".to_string(),
             Item::Literal(literal) => literal.replace('\'', "''").replace('%', "%%"),
             Item::Space(spaces) => spaces.to_string(),
-            _ => bail!("PRQL doesn't support this format specifier"),
+            _ => {
+                return Err(Error::new_simple(
+                    "PRQL doesn't support this format specifier",
+                ))
+            }
         })
     }
 }
@@ -416,7 +439,11 @@ impl DialectHandler for ClickHouseDialect {
                 }
             }
             Item::Space(spaces) => spaces.to_string(),
-            _ => bail!("PRQL doesn't support this format specifier"),
+            _ => {
+                return Err(Error::new_simple(
+                    "PRQL doesn't support this format specifier",
+                ))
+            }
         })
     }
 }
@@ -486,17 +513,23 @@ impl DialectHandler for DuckDbDialect {
             Item::Fixed(Fixed::RFC3339) => "%Y-%m-%dT%H:%M:%S.%fZ".to_string(),
             Item::Literal(literal) => literal.replace('\'', "''").replace('%', "%%"),
             Item::Space(spaces) => spaces.to_string(),
-            _ => bail!("PRQL doesn't support this format specifier"),
+            _ => {
+                return Err(Error::new_simple(
+                    "PRQL doesn't support this format specifier",
+                ))
+            }
         })
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::Dialect;
-    use insta::assert_debug_snapshot;
     use std::str::FromStr;
 
+    use insta::assert_debug_snapshot;
+
+    use super::Dialect;
+
     #[test]
     fn test_dialect_from_str() {
         assert_debug_snapshot!(Dialect::from_str("postgres"), @r###"
