@@ -1,6 +1,4 @@
-mod ast_code_matches;
 mod bad_error_messages;
-mod cli;
 mod dbs;
 mod error_messages;
 mod queries;
